#!/usr/bin/perl

######################################################################
## @brief   convert STAR chimeric format to segemehl-like SAM
## @note    NM and MD tags are not set (use samtools calmd);
##          paired-end data may not be properly converted
## @author  Christian Otto
## @date    Fri Jul  5 15:24:45 CEST 2013
## @version 0.1
######################################################################

while($line = <STDIN>){
    chomp $line; 
    
    do {print $line."\n"; next} if $line =~ /^@/;
    
    @F=split("\t", $line); 
    next if $F[1] & 0x4;
    die "Paired-end data not supported"
	if ($F[1] & 0x1) || ($F[1] & 0x40) || ($F[1] & 0x80);


    $qrypos = 0;
    $qrylen = length($F[9]);
    ## split up fragments by N in CIGAR
    while($F[5] =~ m/\d+N/){

	$F[5] =~ s/^([^N]+[SMIDX=])(\d+)N//;
	$curcigar = $1; 
	$curjump = $2; 
	
	$tmp = $curcigar;
	$clip5 = ($tmp =~ s/^(\d+)S//) ? $1 : 0;
	$clip3 = ($tmp =~ s/(\d+)S$//) ? $1 : 0;
	die if $tmp =~ m/S/;

	$qrypos += $clip5;
	$qryst = $qrypos + 1;
	$refst = $F[3];
	
	$tmp = $curcigar;
	$tmp =~ s/(\d+)[MX=D]/$F[3]+=$1/eg;
	$tmp = $curcigar;
	$tmp =~ s/(\d+)[MX=I]/$qrypos+=$1/eg;

	$qryend = $qrypos;
	$refend = $F[3]-1;
	$F[3] += $curjump;

	## qryst is after soft-clipping (due to the "star-specific"
	## soft-clipping in order to represent chimeric alignments)
	## qryend is after soft-clipping + matching
	## => current qrylen = $qryend-$qryst+1+$clip5 
	$curseq = substr($F[9], 0, $qryend-$qryst+1+$clip5, "");
	$curqual = substr($F[10], 0, $qryend-$qryst+1+$clip5, "");
	
	if ($F[1] & 0x10){
	    $tmp = $qryst;
	    $qryst = $qrylen - $qrypos + 1;
	    $qryend = $qrylen - $tmp + 1;
	}
	
	#print $clip5."\t".$clip3."\t".$qryst."\t".$qryend."\t".$qrylen."\t".$curcigar."\t".$curseq."\t".$F[3]."\t".$F[5]."\n";
	push(@{$READS{$F[0]}}, [$F[0], $F[1], $F[2], $refst, $F[4], $curcigar, $F[6], $F[7], $F[8], 
				$curseq, $curqual, $qryst, $qryend, $qrylen, $refend]);
    }
    $tmp = $F[5];
    $clip5 = ($tmp =~ s/^(\d+)S//) ? $1 : 0;
    $clip3 = ($tmp =~ s/(\d+)S$//) ? $1 : 0;
    die if $tmp =~ m/S/;

    $qrypos += $clip5;
    $qryst = $qrypos + 1;
    $refst = $F[3];
    $refend = $F[3]-1;
    
    $tmp = $F[5];
    $tmp =~ s/(\d+)[MX=D]/$refend+=$1/eg;
    $tmp = $F[5];
    $tmp =~ s/(\d+)[MX=I]/$qrypos+=$1/eg;

    $qryend = $qrypos;
    if ($F[1] & 0x10){
	$tmp = $qryst;
	$qryst = $qrylen - $qrypos + 1;
	$qryend = $qrylen - $tmp + 1;
    }
    #print $clip5."\t".$clip3."\t".$qryst."\t".$qryend."\t".$qrylen."\t".$F[5]."\n";

    push(@{$READS{$F[0]}}, [@F[0..10], $qryst, $qryend, $qrylen, $refend]);
}

## iterate through reads
foreach(sort(keys(%READS))){

    ## sort fragments by qryst
    @frags = sort { $a->[-4] <=> $b->[-4] } @{$READS{$_}};

    ## check whether qry range [qryst,qryend] overlap
    ## with any other qry range in frags
    for ($i = 0; $i < scalar(@frags); $i++){
	#print join("\t", @{$frags[$i]})."\n";
	for ($j = $i + 1; $j < scalar(@frags); $j++){
	    if (($frags[$i]->[-4] <= $frags[$j]->[-4] &&
		 $frags[$i]->[-3] >= $frags[$j]->[-4]) ||
		($frags[$i]->[-4] > $frags[$j]->[-4] &&
		 $frags[$i]->[-4] <= $frags[$j]->[-3])){
		die "Error: overlapping query positions in $_\n";		
	    }
	}
    }
    
    for ($i = 0; $i < scalar(@frags); $i++){
	## correct qry position: soft-clipping at
	## begin of read at first frag and at end
	## of read at last frag is reverted
	if ($i == 0){
	    $frags[$i]->[-4] = 1;
	}
	elsif ($i == scalar(@frags) - 1){
	    $frags[$i]->[-3] = $frags[$i]->[-2];
	}
	@res = @{$frags[$i]}[0..10];
	#splice(@res, -4);
	
	## correct CIGAR, SEQ, QUAL
	## except for cases above:
	## on minus: no soft-clipping
	## at 
	if ($res[1] & 0x10){
	    if ($i != 0 && $res[5] =~ m/\d+S$/){
		$res[5] =~ s/(\d+)S$//;
		$res[9] = substr($res[9], 0, -$1, "");
		$res[10] = substr($res[10], 0, -$1, "");
	    }
	    if ($i != scalar(@frags) - 1 && $res[5] =~ m/^\d+S/){
		$res[5] =~ s/^(\d+)S//;
		$res[9] = substr($res[9], $1);
		$res[10] = substr($res[10], $1);
	    }
	}
	else {
	    if ($i != 0 && $res[5] =~ m/^\d+S/){
		$res[5] =~ s/^(\d+)S//;
		$res[9] = substr($res[9], $1);
		$res[10] = substr($res[10], $1);
	    }
	    if ($i != scalar(@frags) - 1 && $res[5] =~ m/\d+S$/){
		$res[5] =~ s/(\d+)S$//;
		$res[9] = substr($res[9], 0, -$1, "");
		$res[10] = substr($res[10], 0, -$1, "");
	    }
	}

	## set RNEXT, PNEXT to defaults
	$res[6] = "*";
	$res[7] = 0;

	## add NH, XI
	push(@res, "NH:i:1");
	push(@res, "XI:i:0");

	## add XL, XX, XY, and XQ
	push(@res, "XL:i:".scalar(@frags));
	push(@res, "XX:i:".$frags[$i]->[-4]);
	push(@res, "XY:i:".$frags[$i]->[-3]);
	push(@res, "XQ:i:".$i);

	## set remaining segemehl tags (see convTo2Se.pl)
	if ($i != 0){
	    push(@res, "XP:Z:".$frags[$i-1]->[2]);
	    push(@res, "XU:i:".(($frags[$i-1]->[1] & 0x10)?
				$frags[$i-1]->[3] : $frags[$i-1]->[-1]));
	    push(@res, "XS:i:".(($frags[$i-1]->[1] & 0x10) ? 0 : 64));
	}
	if ($i != scalar(@frags) - 1){
	    push(@res, "XC:Z:".$frags[$i+1]->[2]);
	    push(@res, "XV:i:".(($frags[$i+1]->[1] & 0x10) ? 
				$frags[$i+1]->[-1] : $frags[$i+1]->[3]));
	    push(@res, "XT:i:".(($frags[$i+1]->[1] & 0x10) ? 0 : 32));
	}
	print join("\t", @res)."\n";
    }
}
