#!/usr/bin/perl

######################################################################
## @brief   concatenate two segemehl-like SAM output files and correct
##          SAM tags NH & XI accordingly, can be used to combine the
##          normal and chimeric output of STAR which are non-disjunct
##          in some cases, leading to inconsistencies
## @author  Christian Otto
## @date    Fri Aug 23 13:02:13 CEST 2013
## @version 0.1
######################################################################

die "Invalid call" if scalar(@ARGV) != 2;

$FILE1=$ARGV[0];
$FILE2=$ARGV[1];

if ($FILE1 =~ m/\.gz/){
    open(IN, "gunzip ".$FILE1." |") or 
	die "Error in reading $FILE1";
}
else {
    open(IN, "<".$FILE1) or 
	die "Error in reading $FILE1";
}
while(<IN>){
    chomp;
    do {print $_."\n"; next} if /^@/;

    $name = (split("\t", $_))[0];
    $name =~ s/ .*//g;
    $NH = $1 if /NH:i:(\d+)/ or die $name;
    $XI = $1 if /XI:i:(\d+)/ or die $name;
    
    if (!defined $READS{$name}){
	$READS{$name} = [$NH, 0, []];
    }
    else {
	die $name if $READS{$name}->[0] != $NH;
    }
    push(@{$READS{$name}->[2]}, $_);
}
close(IN);

if ($FILE2 =~ m/\.gz/){
    open(IN, "gunzip ".$FILE2." |") or 
	die "Error in reading $FILE2";
}
else {
    open(IN, "<".$FILE2) or 
	die "Error in reading $FILE2";
}
while(<IN>){
    chomp;

    do {print $_."\n"; next} if /^@/;
    $name = (split("\t", $_))[0];
    $name =~ s/ .*//g;
    $NH = $1 if /NH:i:(\d+)/ or die $name;
    $XI = $1 if /XI:i:(\d+)/ or die $name;

    if (!defined $READS{$name}){
	print $_."\n";
    }
    else {
	$READS{$name}->[1] = $NH;

	$XI += $READS{$name}->[0];
	$_ =~ s/XI:i:\d+/XI:i:$XI/;

	push(@{$READS{$name}->[2]}, $_);
    }    
}
close(IN);

foreach(keys(%READS)){
    $NH = $READS{$_}->[0] + $READS{$_}->[1];
    for ($i = 0; $i < scalar(@{$READS{$_}->[2]}); $i++){
	$line = $READS{$_}->[2]->[$i];
	$line =~ s/NH:i:\d+/NH:i:$NH/;
	print $line."\n";
    }
}
